"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const wrappers_1 = require("./wrappers");
const proxies_private_1 = require("./proxies.private");
function magicContainer(containerConfig) {
    const configWithDependencies = mapValues(containerConfig, value => {
        if (isConstant(value)) {
            return new wrappers_1.ConstantValue(value);
        }
        const wrappedValue = isWrapped(value) ? value : new wrappers_1.Funktion(value);
        const dependencyTrackingProxy = new proxies_private_1.DependencyTrackingProxy();
        wrappedValue.trackDeps(dependencyTrackingProxy.proxy);
        wrappedValue.dependencies = Array.from(dependencyTrackingProxy.dependencies);
        return wrappedValue;
    });
    const containerProxy = new proxies_private_1.ContainerProxy(configWithDependencies);
    return containerProxy.proxy;
}
exports.magicContainer = magicContainer;
const isConstant = (v) => typeof v !== "function" && !isWrapped(v);
const isWrapped = (v) => v instanceof wrappers_1.TypeSafeDiWrapper;
function mapValues(v, func) {
    const newValues = Object.keys(v).map(k => ({ key: k, val: func(v[k]) }));
    return newValues.reduce((acc, cur) => (Object.assign({}, acc, { [cur.key]: cur.val })), {});
}
