"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class DependencyTrackingProxy {
    constructor() {
        this.dependencies = new Set();
        this.proxy = new Proxy({}, {
            get: (_target, name) => {
                if (typeof name !== "string") {
                    throw new Error("Only strings are accepted as keys inside container");
                }
                this.dependencies.add(name);
                return undefined;
            },
        });
    }
}
exports.DependencyTrackingProxy = DependencyTrackingProxy;
const RESOLVE_DEPTH_LIMIT = 20;
class ContainerProxy {
    constructor(entities) {
        this.entities = entities;
        this.proxy = new Proxy({}, {
            get: (_target, name) => {
                if (typeof name !== "string") {
                    throw new Error("Only strings are accepted as keys inside container");
                }
                return this.resolve(name, { depth: 1, path: [name] });
            },
        });
    }
    resolve(name, ctx) {
        if (ctx.depth > RESOLVE_DEPTH_LIMIT) {
            throw new Error(`Circular dependency detected: ${ctx.path}`);
        }
        const entity = this.entities[name];
        if (!entity) {
            throw new Error(`${name} is missing in the container`);
        }
        if (entity.dependencies.length === 0) {
            return entity.apply(undefined, ctx);
        }
        const resolvedDeps = entity.dependencies.map(dependencyName => ({
            name: dependencyName,
            value: this.resolve(dependencyName, { depth: ctx.depth + 1, path: [dependencyName, ...ctx.path] }),
        }));
        const miniContainer = resolvedDeps.reduce((acc, current) => (Object.assign({ [current.name]: current.value }, acc)), {});
        return entity.apply(miniContainer, ctx);
    }
}
exports.ContainerProxy = ContainerProxy;
