import { IResolutionCtx } from "./types";
/**
 * Base class for all wrappers
 */
export declare abstract class TypeSafeDiWrapper<T> {
    dependencies: ReadonlyArray<string>;
    constructor(dependencies?: ReadonlyArray<string>);
    abstract apply(container: any, ctx: IResolutionCtx): T;
    abstract trackDeps(proxy: any): void;
}
/**
 * Wraps singleton class.
 */
export declare class Clazz<T extends new (...args: any[]) => any> extends TypeSafeDiWrapper<T> {
    readonly clazz: T;
    constructor(clazz: T, explicitDependencies?: ReadonlyArray<string>);
    apply(container: any): T;
    trackDeps(proxy: any): void;
}
/**
 * Derives constant value from the container. Useful for configs etc.
 */
export declare class DerivedValue<C> extends TypeSafeDiWrapper<C> {
    readonly fn: (d: any) => () => C;
    private cached?;
    constructor(fn: (d: any) => () => C, explicitDependencies?: ReadonlyArray<string>);
    apply(container: any): C;
    trackDeps(proxy: any): void;
}
/**
 * Dynamic function that can use runtime information about container state.
 */
export declare class DynamicFunction<C extends Function> extends TypeSafeDiWrapper<C> {
    readonly value: (d: any, c: IResolutionCtx) => C;
    constructor(value: (d: any, c: IResolutionCtx) => C, explicitDependencies?: ReadonlyArray<string>);
    apply(container: any, ctx: IResolutionCtx): C;
    trackDeps(proxy: any): void;
}
/**
 * Constant value. Default type for everything which is not a function.
 */
export declare class ConstantValue<C> extends TypeSafeDiWrapper<C> {
    readonly value: C;
    constructor(value: C);
    apply(_container: any): C;
    trackDeps(_proxy: any): void;
}
/**
 * Function type. Default for functions in the container.
 */
export declare class Funktion<C extends Function> extends TypeSafeDiWrapper<C> {
    readonly fn: C;
    private cached?;
    constructor(fn: C, explicitDependencies?: ReadonlyArray<string>);
    apply(container: any): C;
    trackDeps(proxy: any): void;
}
