"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Base class for all wrappers
 */
class TypeSafeDiWrapper {
    constructor(dependencies = []) {
        this.dependencies = dependencies;
    }
}
exports.TypeSafeDiWrapper = TypeSafeDiWrapper;
/**
 * Wraps singleton class.
 */
class Clazz extends TypeSafeDiWrapper {
    constructor(clazz, explicitDependencies) {
        super(explicitDependencies);
        this.clazz = clazz;
    }
    apply(container) {
        return new this.clazz(container);
    }
    trackDeps(proxy) {
        new this.clazz(proxy);
    }
}
exports.Clazz = Clazz;
/**
 * Derives constant value from the container. Useful for configs etc.
 */
class DerivedValue extends TypeSafeDiWrapper {
    constructor(fn, explicitDependencies) {
        super(explicitDependencies);
        this.fn = fn;
    }
    apply(container) {
        if (!this.cached) {
            this.cached = this.fn(container)();
        }
        return this.cached;
    }
    trackDeps(proxy) {
        this.fn(proxy);
    }
}
exports.DerivedValue = DerivedValue;
/**
 * Dynamic function that can use runtime information about container state.
 */
class DynamicFunction extends TypeSafeDiWrapper {
    constructor(value, explicitDependencies) {
        super(explicitDependencies);
        this.value = value;
    }
    apply(container, ctx) {
        return this.value(container, ctx);
    }
    trackDeps(proxy) {
        this.value(proxy, undefined);
    }
}
exports.DynamicFunction = DynamicFunction;
/**
 * Constant value. Default type for everything which is not a function.
 */
class ConstantValue extends TypeSafeDiWrapper {
    constructor(value) {
        super();
        this.value = value;
    }
    apply(_container) {
        return this.value;
    }
    trackDeps(_proxy) { }
}
exports.ConstantValue = ConstantValue;
/**
 * Function type. Default for functions in the container.
 */
class Funktion extends TypeSafeDiWrapper {
    constructor(fn, explicitDependencies) {
        super(explicitDependencies);
        this.fn = fn;
    }
    apply(container) {
        if (!this.cached) {
            this.cached = this.fn(container);
        }
        return this.cached;
    }
    trackDeps(proxy) {
        return this.fn(proxy);
    }
}
exports.Funktion = Funktion;
